@@ -4,7 +4,10 @@ callbacks = []
 
 def subscribe(type, callback):
     callbacks.append((type, callback))
-    
+
+def unsubscribe(type, callback):
+    callbacks.remove((type, callback))
+
 def notify(event):
     for type, callback in callbacks:
         if isinstance(event, type):
@@ -19,6 +22,7 @@ class Event:
 
 class ProcessLogEvent(Event):
     """ Abstract """
+    channel = None
     def __init__(self, process, pid, data):
         self.process = process
         self.pid = pid
@@ -87,7 +91,7 @@ class SupervisorRunningEvent(SupervisorStateChangeEvent):
 class SupervisorStoppingEvent(SupervisorStateChangeEvent):
     pass
 
-class EventRejectedEvent: # purposely does not subclass Event 
+class EventRejectedEvent: # purposely does not subclass Event
     def __init__(self, process, event):
         self.process = process
         self.event = event
@@ -199,7 +203,7 @@ class EventTypes:
     PROCESS_COMMUNICATION_STDERR = ProcessCommunicationStderrEvent
     PROCESS_LOG = ProcessLogEvent
     PROCESS_LOG_STDOUT = ProcessLogStdoutEvent
-    PROCESS_LOG_STDERR = ProcessLogStderrEvent     
+    PROCESS_LOG_STDERR = ProcessLogStderrEvent
     REMOTE_COMMUNICATION = RemoteCommunicationEvent
     SUPERVISOR_STATE_CHANGE = SupervisorStateChangeEvent # abstract
     SUPERVISOR_STATE_CHANGE_RUNNING = SupervisorRunningEvent
@@ -218,4 +222,4 @@ def getEventNameByType(requested):
             return name
 
 def register(name, event):
-    EventTypes.__dict__[name] = event
+    setattr(EventTypes, name, event)
