@@ -62,6 +62,7 @@ class DummyOptions:
         self.existing = []
         self.openreturn = None
         self.readfd_result = ''
+        self.parse_criticals = []
         self.parse_warnings = []
         self.parse_infos = []
         self.serverurl = 'http://localhost:9001'
@@ -86,13 +87,13 @@ class DummyOptions:
     def cleanup_fds(self):
         self.fds_cleaned_up = True
 
-    def set_rlimits(self):
+    def set_rlimits_or_exit(self):
         self.rlimits_set = True
-        return ['rlimits_set']
+        self.parse_infos.append('rlimits_set')
 
-    def set_uid(self):
+    def set_uid_or_exit(self):
         self.setuid_called = True
-        return 'setuid_called'
+        self.parse_criticals.append('setuid_called')
 
     def openhttpservers(self, supervisord):
         self.httpservers_opened = True
@@ -109,8 +110,8 @@ class DummyOptions:
     def get_socket_map(self):
         return self.socket_map
 
-    def make_logger(self, critical_msgs, warn_msgs, info_msgs):
-        self.make_logger_messages = critical_msgs, warn_msgs, info_msgs
+    def make_logger(self):
+        pass
 
     def clear_autochildlogdir(self):
         self.autochildlogdir_cleared = True
@@ -203,7 +204,7 @@ class DummyOptions:
         self.execv_args = (filename, argv)
         self.execv_environment = environment
 
-    def dropPrivileges(self, uid):
+    def drop_privileges(self, uid):
         if self.setuid_msg:
             return self.setuid_msg
         self.privsdropped = uid
@@ -525,6 +526,9 @@ class DummyPConfig:
         self.autochildlogs_created = False
         self.serverurl = serverurl
 
+    def get_path(self):
+        return ["/bin", "/usr/bin", "/usr/local/bin"]
+
     def create_autochildlogs(self):
         self.autochildlogs_created = True
 
@@ -565,7 +569,7 @@ def makeExecutable(file, substitutions=None):
     f = open(tmpnam, 'w')
     f.write(data)
     f.close()
-    os.chmod(tmpnam, 0755)
+    os.chmod(tmpnam, int('755', 8))
     return tmpnam
 
 def makeSpew(unkillable=False):
@@ -1014,10 +1018,14 @@ class DummyProcessGroup:
         self.all_stopped = False
         self.dispatchers = {}
         self.unstopped_processes = []
+        self.before_remove_called = False
 
     def transition(self):
         self.transitioned = True
 
+    def before_remove(self):
+        self.before_remove_called = True
+
     def stop_all(self):
         self.all_stopped = True
 
@@ -1132,6 +1140,7 @@ class DummyEvent:
 class DummyPoller:
     def __init__(self, options):
         self.result = [], []
+        self.closed = False
 
     def register_readable(self, fd):
         pass
@@ -1142,6 +1151,9 @@ class DummyPoller:
     def poll(self, timeout):
         return self.result
 
+    def close(self):
+        self.closed = True
+
 def dummy_handler(event, result):
     pass
 
