@@ -1,5 +1,6 @@
 import os
 import signal
+import tempfile
 import time
 import unittest
 import sys
@@ -165,6 +166,19 @@ class SubprocessTests(unittest.TestCase):
         self.assertEqual(args[0], '/bin/sh')
         self.assertEqual(args[1], ['sh', 'foo'])
 
+    def test_get_execv_args_rel_searches_using_pconfig_path(self):
+        f = tempfile.NamedTemporaryFile()
+        dirname, basename = os.path.split(f.name)
+        executable = '%s foo' % basename
+        options = DummyOptions()
+        config = DummyPConfig(options, 'sh', executable)
+        config.get_path = lambda: [ dirname ]
+        instance = self._makeOne(config)
+        args = instance.get_execv_args()
+        self.assertEqual(args[0], f.name)
+        self.assertEqual(args[1], [basename, 'foo'])
+        f.close()
+
     def test_record_spawnerr(self):
         options = DummyOptions()
         config = DummyPConfig(options, 'test', '/test')
@@ -404,13 +418,13 @@ class SubprocessTests(unittest.TestCase):
     def test_spawn_as_child_sets_umask(self):
         options = DummyOptions()
         options.forkpid = 0
-        config = DummyPConfig(options, 'good', '/good/filename', umask=002)
+        config = DummyPConfig(options, 'good', '/good/filename', umask=2)
         instance = self._makeOne(config)
         result = instance.spawn()
         self.assertEqual(result, None)
         self.assertEqual(options.execv_args,
                          ('/good/filename', ['/good/filename']) )
-        self.assertEqual(options.umaskset, 002)
+        self.assertEqual(options.umaskset, 2)
         self.assertEqual(options.execve_called, True)
         # if the real execve() succeeds, the code that writes the
         # "was not spawned" message won't be reached.  this assertion
@@ -633,10 +647,10 @@ class SubprocessTests(unittest.TestCase):
         # this is a functional test
         from supervisor.tests.base import makeSpew
         try:
-            called = 0
-            def foo(*args):
-                called = 1
-            signal.signal(signal.SIGCHLD, foo)
+            sigchlds = []
+            def sighandler(*args):
+                sigchlds.append(True)
+            signal.signal(signal.SIGCHLD, sighandler)
             executable = makeSpew()
             options = DummyOptions()
             config = DummyPConfig(options, 'spew', executable)
@@ -664,6 +678,7 @@ class SubprocessTests(unittest.TestCase):
             pid, sts = os.waitpid(-1, os.WNOHANG)
             data = os.popen('ps').read()
             self.assertEqual(data.find(repr(origpid)), -1) # dubious
+            self.assertNotEqual(sigchlds, [])
         finally:
             try:
                 os.remove(executable)
@@ -1642,6 +1657,16 @@ class ProcessGroupBaseTests(unittest.TestCase):
         unstopped = group.get_unstopped_processes()
         self.assertEqual(unstopped, [process1])
 
+    def test_before_remove(self):
+        options = DummyOptions()
+        from supervisor.states import ProcessStates
+        pconfig1 = DummyPConfig(options, 'process1', 'process1','/bin/process1')
+        process1 = DummyProcess(pconfig1, state=ProcessStates.STOPPING)
+        gconfig = DummyPGroupConfig(options, pconfigs=[pconfig1])
+        group = self._makeOne(gconfig)
+        group.processes = { 'process1': process1 }
+        group.before_remove()  # shouldn't raise
+
     def test_stop_all(self):
         from supervisor.states import ProcessStates
         options = DummyOptions()
@@ -1818,6 +1843,18 @@ class EventListenerPoolTests(ProcessGroupBaseTests):
             (events.EventRejectedEvent, pool.handle_rejected))
         self.assertEqual(pool.serial, -1)
 
+    def test_before_remove_unsubscribes_from_events(self):
+        options = DummyOptions()
+        gconfig = DummyPGroupConfig(options)
+        class EventType:
+            pass
+        gconfig.pool_events = (EventType,)
+        pool = self._makeOne(gconfig)
+        from supervisor import events
+        self.assertEqual(len(events.callbacks), 2)
+        pool.before_remove()
+        self.assertEqual(len(events.callbacks), 0)
+
     def test__eventEnvelope(self):
         options = DummyOptions()
         options.identifier = 'thesupervisorname'
@@ -1901,7 +1938,7 @@ class EventListenerPoolTests(ProcessGroupBaseTests):
         self.assertEqual(process1.listener_state, EventListenerStates.READY)
         self.assertEqual(pool.event_buffer, [event])
         self.assertEqual(options.logger.data[0],
-                         'rebuffering event abc for pool whatever (bufsize 0)')
+            'rebuffering event abc for pool whatever (buf size=0, max=10)')
 
     def test__acceptEvent_attaches_pool_serial_and_serial(self):
         from supervisor.process import GlobalSerial
